# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2023 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Static pages module for Invenio."""

from .ext import InvenioPages, InvenioPagesREST
from .records.models import PageList, PageModel

__version__ = "3.2.0"

__all__ = ("__version__", "InvenioPages", "InvenioPagesREST", "PageModel", "PageList")



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test views for Pages module."""

from unittest import mock

import pytest
from invenio_db import db
from jinja2.exceptions import UndefinedError
from werkzeug.exceptions import NotFound

from invenio_pages import PageModel as Page
from invenio_pages.views import render_page


@pytest.mark.skip(reason="views to be refactored after discussion with architects")
def test_page_content(pages_fixture, app, base_client):
    """Test page content."""
    resp = base_client.get("/dogs/shiba")
    assert resp.status_code == 200
    assert "so doge!" in str(resp.get_data())
    assert "so doge!" in str(render_page("/dogs/shiba"))


@pytest.mark.skip(reason="views to be refactored after discussion with architects")
def test_page_content_dynamic(pages_fixture, app):
    """Test page content."""
    app.config["PAGES_WHITELIST_CONFIG_KEYS"] = ["MYVAR"]
    content = "dynamic-content"
    with app.app_context():
        app.config["MYVAR"] = content
        page = Page(
            url="/dynamic",
            title="Dynamic page",
            content="{{MYVAR}}",
            template_name="invenio_pages/dynamic.html",
        )
        db.session.add(page)
        db.session.commit()

        with app.test_request_context("/dynamic"):
            assert content in render_page("/dynamic")

        with app.test_client() as client:
            resp = client.get("/dynamic")
            assert resp.status_code == 200
            assert content in resp.get_data(as_text=True)


@pytest.mark.skip(reason="views to be refactored after discussion with architects")
def test_current_app_and_config_not_visible(pages_fixture, app):
    """Test current_app and config are not visible to template."""
    app.config.update(SECRET_KEY="super secret")
    with app.app_context():
        page = Page(
            url="/dynamic",
            title="Dynamic page",
            content="{{SECRET_KEY}}",
            template_name="invenio_pages/dynamic.html",
        )
        db.session.add(page)
        db.session.commit()

        with app.test_request_context("/dynamic"):
            assert app.config["SECRET_KEY"] not in render_page("/dynamic")

            page.content = "{{config.SECRET_KEY}}"
            db.session.commit()
            with pytest.raises(UndefinedError):
                render_page("/dynamic")

            page.content = "{{current_app.config['SECRET_KEY']}}"
            db.session.commit()
            with pytest.raises(UndefinedError):
                render_page("/dynamic")


@pytest.mark.skip(reason="views to be refactored after discussion with architects")
def test_non_existing_page(pages_fixture, app):
    """Test non-existing page content."""

    # render_page function
    with app.test_request_context("/invalid/url/errors"):
        with pytest.raises(NotFound):
            render_page("/invalid/url/errors")


@pytest.mark.skip(reason="views to be refactored after discussion with architects")
def test_runtime_added_page(pages_fixture, app):
    """Test runtime added page."""

    with app.test_request_context("/runtime/added"):
        with app.test_client() as client:
            resp = client.get("/runtime/added")
            assert resp.status_code == 404

            new_page = Page(
                url="/runtime/added",
                title="Runtime added page!",
                content="added after initial page mapping.",
                template_name="invenio_pages/default.html",
            )
            db.session.add(new_page)
            db.session.commit()

            resp = client.get("/runtime/added")
            assert resp.status_code == 200
            assert "added after initial page mapping." in str(resp.get_data())


@pytest.mark.skip(reason="views to be refactored after discussion with architects")
def test_pre_existing_404_function(pages_fixture, app):
    """Test pre existing 404."""
    with app.test_request_context("/runtime/added"):
        with app.test_client() as client:
            resp = client.get("/runtime/added")
            assert resp.status_code == 404

            new_page = Page(
                url="/runtime/added",
                title="Runtime added page!",
                content="added after initial page mapping.",
                template_name="invenio_pages/default.html",
            )
            db.session.add(new_page)
            db.session.commit()

            resp = client.get("/runtime/added")
            assert resp.status_code == 200
            assert "added after initial page mapping." in str(resp.get_data())



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Models for Pages module."""

import pytest
from invenio_records_resources.services.base.utils import map_search_params

from invenio_pages import PageModel as Page
from invenio_pages.records.errors import PageNotCreatedError, PageNotFoundError
from invenio_pages.services.config import PageServiceConfig


def test_page_repr(module_scoped_pages_fixture, base_app):
    dog_page = Page.get_by_url("/dogs/shiba")
    assert dog_page.__repr__() == "URL: /dogs/shiba, title: Page for doge!"


def test_page_versions(module_scoped_pages_fixture, base_app, db):
    dog_page = Page.get_by_url("/dogs")
    dog_page.title = "Just a dog!"
    db.session.commit()

    dog_page = Page.get_by_url("/dogs")
    assert "Just a dog!" == dog_page.title
    assert 2 == dog_page.versions.count()
    assert "Page for Dogs!" == dog_page.versions[0].title


def test_page_versions(module_scoped_pages_fixture, base_app, db):
    dog_page = Page.get_by_url("/dogs")
    dog_page.title = "Just a dog!"
    db.session.commit()

    dog_page = Page.get_by_url("/dogs")
    assert "Just a dog!" == dog_page.title
    assert 2 == dog_page.versions.count()
    assert "Page for Dogs!" == dog_page.versions[0].title


def test_read(module_scoped_pages_fixture, base_app):
    assert Page.get(1).title == "Page for Dogs!"
    with pytest.raises(PageNotFoundError):
        Page.get(10)


def test_search(module_scoped_pages_fixture, base_app):
    pages = Page.search(map_search_params(PageServiceConfig.search, {}), [])
    assert len(pages.items) == 4

    pages = Page.search(map_search_params(PageServiceConfig.search, {"size": 2}), [])
    assert len(pages.items) == 2

    pages = Page.search(
        map_search_params(PageServiceConfig.search, {"sort": "title"}), []
    )
    assert pages.items[0].id == 3

    pages = Page.search(
        map_search_params(
            PageServiceConfig.search,
            {"sort": "title", "sort_direction": "desc"},
        ),
        [],
    )
    assert pages.items[0].id == 4

    pages = Page.search(
        map_search_params(PageServiceConfig.search, {"sort_direction": "desc"}), []
    )
    assert pages.items[0].id == 4

    pages = Page.search(
        map_search_params(
            PageServiceConfig.search,
            {"size": 3, "sort": "title", "sort_direction": "desc"},
        ),
        [],
    )
    assert pages.items[0].id == 4

    pages = Page.search(
        map_search_params(PageServiceConfig.search, {"sort": "url"}), []
    )
    assert pages.items[0].id == 3
    assert pages.items[3].id == 4


def test_create(module_scoped_pages_fixture, base_app):
    data = {
        "url": "/astures",
        "title": "Astures",
        "content": "Astures",
        "description": "Los astures (astures en latín) fueron un grupo de pueblos celtas...",
        "template_name": "invenio_pages/default.html",
    }
    page = Page.create(data)
    assert page.title == "Astures"

    id = page.id
    assert Page.get(id).title == "Astures"

    with pytest.raises(PageNotCreatedError):
        Page.create(data)


def test_delete(module_scoped_pages_fixture, base_app):
    data = {
        "url": "/cantabros",
        "title": "Cantabros",
        "content": "Cantabros",
        "description": "El término cántabros...",
        "template_name": "invenio_pages/default.html",
    }
    page = Page.create(data)
    id = page.id
    assert Page.get(id).title == "Cantabros"
    Page.delete(page)
    with pytest.raises(PageNotFoundError):
        Page.get(id)


def test_update(module_scoped_pages_fixture, base_app):
    data = {
        "url": "/lusitanos",
        "title": "Lusitanos",
        "content": "Lusitanos",
        "description": "El término lusitanos...",
    }
    assert Page.get(1).title != "Lusitanos"
    Page.update(data, 1)
    page = Page.get(1)
    assert page.title == "Lusitanos"
    assert page.template_name == "invenio_pages/default.html"


def test_delete_all(module_scoped_pages_fixture, base_app):
    Page.delete_all()
    pages = Page.search(map_search_params(PageServiceConfig.search, {}), [])
    assert len(pages.items) == 0



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.


"""Pytest configuration."""
import pytest
from invenio_access.models import ActionRoles
from invenio_access.permissions import superuser_access
from invenio_accounts.models import Role
from invenio_administration.permissions import administration_access_action
from invenio_app.factory import create_api

from invenio_pages import PageModel as Page


@pytest.fixture(scope="module")
def app(base_app, database):
    """Invenio application with only database.

    Scope: module
    """
    yield base_app


@pytest.fixture(scope="module")
def create_app(instance_path, entry_points):
    """Application factory fixture."""
    return create_api


@pytest.fixture(scope="function")
def pages_fixture(app, db):
    """Page fixtures.

    Scope: function
    """
    pages = [
        Page(
            url="/dogs",
            title="Page for Dogs!",
            content="Generic dog.",
            template_name="invenio_pages/default.html",
        ),
        Page(
            url="/dogs/shiba",
            title="Page for doge!",
            content="so doge!",
            template_name="invenio_pages/default.html",
        ),
        Page(
            url="/cows/",
            title="Page for Cows!",
            content="Generic cow.",
            template_name="invenio_pages/default.html",
        ),
        Page(
            url="/htmldog",
            title="Page for modern dogs!",
            content="<h1>HTML aware dog.</h1>.\n" '<p class="test">paragraph<br /></p>',
            template_name="invenio_pages/default.html",
        ),
    ]
    for page in pages:
        db.session.add(page)
    db.session.commit()


@pytest.fixture(scope="module")
def module_scoped_pages_fixture(app, database):
    """Page fixtures.

    Scope: module
    """
    db = database
    pages = [
        Page(
            url="/dogs",
            title="Page for Dogs!",
            content="Generic dog.",
            template_name="invenio_pages/default.html",
        ),
        Page(
            url="/dogs/shiba",
            title="Page for doge!",
            content="so doge!",
            template_name="invenio_pages/default.html",
        ),
        Page(
            url="/cows/",
            title="Page for Cows!",
            content="Generic cow.",
            template_name="invenio_pages/default.html",
        ),
        Page(
            url="/htmldog",
            title="Page for modern dogs!",
            content="<h1>HTML aware dog.</h1>.\n" '<p class="test">paragraph<br /></p>',
            template_name="invenio_pages/default.html",
        ),
    ]
    for page in pages:
        db.session.add(page)
    db.session.commit()


@pytest.fixture(scope="function")
def superuser_role_need(db):
    """Store 1 role with 'superuser-access' ActionNeed."""
    role = Role(name="superuser-access")
    db.session.add(role)

    action_role = ActionRoles.create(action=superuser_access, role=role)
    db.session.add(action_role)

    db.session.commit()
    return action_role.need


@pytest.fixture()
def superuser(UserFixture, app, db, superuser_role_need):
    """Admin user for requests."""
    u = UserFixture(
        email="superuser@inveniosoftware.org",
        password="superuser",
    )
    u.create(app, db)

    datastore = app.extensions["security"].datastore
    _, role = datastore._prepare_role_modify_args(u.user, "superuser-access")

    datastore.add_role_to_user(u.user, role)
    db.session.commit()
    return u


@pytest.fixture()
def superuser_identity(superuser):
    """Superuser identity fixture."""
    identity = superuser.identity
    return identity


@pytest.fixture(scope="function")
def admin_role_need(db):
    """Store 1 role with 'administration-access' ActionNeed."""
    role = Role(name="administration-access")
    db.session.add(role)

    action_role = ActionRoles.create(action=administration_access_action, role=role)
    db.session.add(action_role)

    db.session.commit()
    return action_role.need


@pytest.fixture()
def admin(UserFixture, app, db, admin_role_need):
    """Admin user for requests."""
    u = UserFixture(
        email="admin@inveniosoftware.org",
        password="admin",
    )
    u.create(app, db)

    datastore = app.extensions["security"].datastore
    _, role = datastore._prepare_role_modify_args(u.user, "administration-access")

    datastore.add_role_to_user(u.user, role)
    db.session.commit()
    return u


@pytest.fixture()
def admin_user_identity(admin):
    """Simple identity fixture."""
    identity = admin.identity
    return identity


@pytest.fixture()
def user(UserFixture, app, db):
    """General user for requests."""
    user = UserFixture(email="user1@example.org", password="user1")
    user.create(app, db)

    return user


@pytest.fixture()
def simple_user_identity(user):
    """Simple identity fixture."""
    identity = user.identity
    return identity



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
# Copyright (C) 2023 KTH Royal Institute of Technology.
#
# Invenio-Pages is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Pages schema."""

from datetime import timezone

from flask import current_app
from marshmallow import Schema, fields
from marshmallow_utils.fields import SanitizedHTML, TZDateTime


class DynamicSanitizedHTML(SanitizedHTML):
    """A subclass of SanitizedHTML that dynamically configures allowed HTML tags and attributes based on application settings."""

    def __init__(self, *args, **kwargs):
        """Initializes DynamicSanitizedHTML with dynamic tag and attribute settings."""
        super().__init__(tags=None, attrs=None, *args, **kwargs)

    def _deserialize(self, value, attr, data, **kwargs):
        """Deserialize value with dynamic HTML tags and attributes based on Flask app context or defaults."""
        self.tags = (
            current_app.config.get("ALLOWED_HTML_TAGS", [])
            + current_app.config["PAGES_ALLOWED_EXTRA_HTML_TAGS"]
        )
        self.attrs = self.attrs = dict(
            **current_app.config.get("ALLOWED_HTML_ATTRS", {}),
            **current_app.config["PAGES_ALLOWED_EXTRA_HTML_ATTRS"]
        )

        return super()._deserialize(value, attr, data, **kwargs)


class PageSchema(Schema):
    """Schema for page."""

    id = fields.String()
    url = fields.String(metadata={"create_only": True})
    title = fields.String()
    content = DynamicSanitizedHTML()
    description = fields.String()
    template_name = fields.String()
    created = TZDateTime(timezone=timezone.utc, format="iso", dump_only=True)
    updated = TZDateTime(timezone=timezone.utc, format="iso", dump_only=True)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Hijacks `mock` to fake as many non-available modules as possible."""
import sys
import types

try:
    import unittest.mock as mock
except ImportError:
    import mock

# skip `_is_magic` check.
orig_is_magic = mock._is_magic


def always_false(*args, **kwargs):
    """Returns false."""
    return False


# avoid spec configuration for mocked classes with super classes.
# honestly this does not happen very often and is kind of a tricky case.
orig_mock_add_spec = mock.NonCallableMock._mock_add_spec


def mock_add_spec_fake(self, spec, spec_set):
    """."""
    orig_mock_add_spec(self, None, None)


class MyMagicMock(mock.MagicMock):
    """Special MagicMock with empty docs."""


class SuperMockMetaMeta(MyMagicMock):
    """Set up a fake class-metaclass hierarchy."""

    __metaclass__ = MyMagicMock()


class SuperMockMeta(MyMagicMock):
    """."""

    __metaclass__ = SuperMockMetaMeta


class SuperMock(MyMagicMock):
    """."""

    __metaclass__ = SuperMockMeta


class MockedModule(types.ModuleType):
    """."""

    def __init__(self, name):
        """."""
        super(types.ModuleType, self).__init__(name)
        self.__name__ = super.__name__
        self.__file__ = self.__name__.replace(".", "/") + ".py"
        sys.modules[self.__name__] = self

    def __getattr__(self, key):
        """."""
        obj = SuperMock
        setattr(self, key, obj)
        return obj


# overwrite imports
orig_import = __import__


def import_mock(name, *args, **kwargs):
    """."""
    try:
        return orig_import(name, *args, **kwargs)
    except ImportError:
        return MockedModule(name)


import_patch = mock.patch("__builtin__.__import__", side_effect=import_mock)


# public methods
def activate():
    """."""
    mock._is_magic = always_false
    mock.NonCallableMock._mock_add_spec = mock_add_spec_fake
    import_patch.start()


def deactivate():
    """."""
    import_patch.stop()
    mock.NonCallableMock._mock_add_spec = orig_mock_add_spec
    mock._is_magic = orig_is_magic



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test alembic recipes for Invenio-pages."""

import pytest
from invenio_db.utils import alembic_test_context, drop_alembic_version_table


def test_alembic(base_app, db):
    """Test alembic recipes."""
    ext = base_app.extensions["invenio-db"]

    if db.engine.name == "sqlite":
        raise pytest.skip("Upgrades are not supported on SQLite.")

    base_app.config["ALEMBIC_CONTEXT"] = alembic_test_context()
    # Check that this package's SQLAlchemy models have been properly registered
    tables = [x.name for x in db.get_tables_for_bind()]

    assert "pages_page" in tables
    assert "pages_pagelist" in tables
    assert "pages_page_version" in tables
    assert "pages_pagelist_version" in tables
    # Check that Alembic agrees that there's no further tables to create.
    assert len(ext.alembic.compare_metadata()) == 0

    # Drop everything and recreate tables all with Alembic
    db.drop_all()
    drop_alembic_version_table()
    ext.alembic.upgrade()
    assert len(ext.alembic.compare_metadata()) == 0

    # Try to upgrade and downgrade
    ext.alembic.stamp()
    ext.alembic.downgrade(target="96e796392533")
    ext.alembic.upgrade()
    assert len(ext.alembic.compare_metadata()) == 0

    drop_alembic_version_table()



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022 CERN.
# Copyright (C) 2023 KTH Royal Institute of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

import pytest
from flask import Flask, current_app
from invenio_records_resources.services.errors import PermissionDeniedError

from invenio_pages import InvenioPages
from invenio_pages.config import (
    PAGES_ALLOWED_EXTRA_HTML_ATTRS,
    PAGES_ALLOWED_EXTRA_HTML_TAGS,
)
from invenio_pages.proxies import current_pages_service
from invenio_pages.records.errors import PageNotCreatedError, PageNotFoundError
from invenio_pages.services.schemas import DynamicSanitizedHTML


def test_page_read(module_scoped_pages_fixture, simple_user_identity):
    """Test read service function."""
    page = current_pages_service.read(simple_user_identity, 1).data
    page.pop("created")
    page.pop("updated")
    expected_data = {
        "title": "Page for Dogs!",
        "description": "",
        "url": "/dogs",
        "content": "Generic dog.",
        "id": "1",
        "template_name": "invenio_pages/default.html",
        "links": {"self": "https://127.0.0.1:5000/api/pages/1"},
    }
    assert page == expected_data


def test_page_read_by_url(module_scoped_pages_fixture, simple_user_identity):
    """Test read_by_url service function."""
    page = current_pages_service.read_by_url(simple_user_identity, "/dogs").data
    page.pop("created")
    page.pop("updated")
    expected_data = {
        "title": "Page for Dogs!",
        "description": "",
        "url": "/dogs",
        "content": "Generic dog.",
        "id": "1",
        "template_name": "invenio_pages/default.html",
        "links": {"self": "https://127.0.0.1:5000/api/pages/1"},
    }
    assert page == expected_data


def test_search(module_scoped_pages_fixture, simple_user_identity):
    """Test search service function."""
    pages = current_pages_service.search(simple_user_identity)
    assert pages.total == 4

    pages = current_pages_service.search(simple_user_identity, {"size": 2})
    assert pages.total == 2

    pages = current_pages_service.search(simple_user_identity, {"sort": "title"})
    assert pages.to_dict()["hits"]["hits"][0]["id"] == "3"

    pages = current_pages_service.search(
        simple_user_identity, {"sort": "title", "sort_direction": "desc"}
    )
    assert pages.to_dict()["hits"]["hits"][0]["id"] == "4"

    pages = current_pages_service.search(
        simple_user_identity, {"sort_direction": "desc"}
    )

    assert pages.to_dict()["hits"]["hits"][0]["id"] == "4"

    pages = current_pages_service.search(
        simple_user_identity, {"size": 3, "sort": "title", "sort_direction": "desc"}
    )
    assert pages.to_dict()["hits"]["hits"][0]["id"] == "4"

    pages = current_pages_service.search(simple_user_identity, {"sort": "url"})
    assert pages.to_dict()["hits"]["hits"][0]["id"] == "3"
    assert pages.to_dict()["hits"]["hits"][3]["id"] == "4"

    pages = current_pages_service.search(simple_user_identity, {"q": "Generic dog"})
    assert pages.total == 1

    pages = current_pages_service.search(
        simple_user_identity, {"q": "dog", "sort_direction": "desc"}
    )
    assert pages.total == 3
    assert pages.to_dict()["hits"]["hits"][0]["id"] == "4"


def test_create(module_scoped_pages_fixture, superuser_identity):
    data = {
        "url": "/astures",
        "title": "Astures",
        "content": "Astures",
        "description": "Los astures (astures en latín) fueron un grupo de pueblos celtas...",
        "template_name": "invenio_pages/default.html",
    }
    page = current_pages_service.create(superuser_identity, data)
    assert page["title"] == "Astures"

    id = page["id"]
    assert current_pages_service.read(superuser_identity, id)["title"] == "Astures"

    with pytest.raises(PageNotCreatedError):
        current_pages_service.create(superuser_identity, data)


def test_delete(module_scoped_pages_fixture, superuser_identity):
    data = {
        "url": "/cantabros",
        "title": "Cantabros",
        "content": "Cantabros",
        "description": "El término cántabros...",
        "template_name": "invenio_pages/default.html",
    }
    page = current_pages_service.create(superuser_identity, data)
    id = page["id"]
    assert current_pages_service.read(superuser_identity, id)["title"] == "Cantabros"
    current_pages_service.delete(superuser_identity, page["id"])
    with pytest.raises(PageNotFoundError):
        current_pages_service.read(superuser_identity, id)


def test_update(module_scoped_pages_fixture, admin_user_identity):
    data = {
        "url": "/lusitanos",
        "title": "Lusitanos",
        "content": "Lusitanos",
        "description": "El término lusitanos...",
    }
    assert current_pages_service.read(admin_user_identity, 1)["title"] != "Lusitanos"
    current_pages_service.update(admin_user_identity, data, 1)

    page = current_pages_service.read(admin_user_identity, 1)
    assert page["title"] == "Lusitanos"
    assert page["template_name"] == "invenio_pages/default.html"


def test_update_denied(module_scoped_pages_fixture, simple_user_identity):
    data = {
        "url": "/lusitanos",
        "title": "Lusitanos",
        "content": "Lusitanos",
        "description": "El término lusitanos...",
    }
    with pytest.raises(PermissionDeniedError):
        current_pages_service.update(simple_user_identity, data, 1)


def test_create_denied(module_scoped_pages_fixture, admin_user_identity):
    data = {
        "url": "/arevacos",
        "title": "Arévacos",
        "content": "Arévacos",
        "description": "Los término arévacos",
        "template_name": "invenio_pages/default.html",
    }
    with pytest.raises(PermissionDeniedError):
        page = current_pages_service.create(admin_user_identity, data)


def test_delete_denied(
    module_scoped_pages_fixture, superuser_identity, admin_user_identity
):
    data = {
        "url": "/numantinos",
        "title": "Numantinos",
        "content": "Numantinos",
        "description": "El término numantinos...",
        "template_name": "invenio_pages/default.html",
    }
    page = current_pages_service.create(superuser_identity, data)
    with pytest.raises(PermissionDeniedError):
        current_pages_service.delete(admin_user_identity, page["id"])


def test_delete_all(module_scoped_pages_fixture, superuser_identity):
    current_pages_service.delete_all(superuser_identity)
    pages = current_pages_service.search(superuser_identity)
    assert pages.total == 0


def test_extra_allowed_html_tags():
    """Test instance folder loading."""
    app = Flask("testapp")
    InvenioPages(app)

    assert (
        app.config["PAGES_ALLOWED_EXTRA_HTML_ATTRS"] == PAGES_ALLOWED_EXTRA_HTML_ATTRS
    )
    assert app.config["PAGES_ALLOWED_EXTRA_HTML_TAGS"] == PAGES_ALLOWED_EXTRA_HTML_TAGS

    app.config["PAGES_ALLOWED_EXTRA_HTML_ATTRS"] = ["a"]
    app.config["PAGES_ALLOWED_EXTRA_HTML_TAGS"] = ["a"]
    InvenioPages(app)
    assert app.config["PAGES_ALLOWED_EXTRA_HTML_ATTRS"] == ["a"]
    assert app.config["PAGES_ALLOWED_EXTRA_HTML_TAGS"] == ["a"]


def test_dynamic_sanitized_html_initialization():
    """
    Test the initialization of the DynamicSanitizedHTML class.

    This test verifies that the default values for 'tags' and 'attrs'
    attributes of a DynamicSanitizedHTML instance are set to None.
    It asserts that both these attributes are None upon initialization,
    ensuring that the class starts with no predefined allowed tags or attributes.
    """
    html_sanitizer = DynamicSanitizedHTML()
    assert html_sanitizer.tags is None
    assert html_sanitizer.attrs is None


def test_dynamic_sanitized_html(app):
    """
    Tests DynamicSanitizedHTML with custom tags and attributes in an app context.
    Verifies if custom settings are properly applied and reflected in the output.
    """
    with app.app_context():
        # Set up the extra configuration
        current_app.config["PAGES_ALLOWED_EXTRA_HTML_TAGS"] = ["customtag"]
        current_app.config["PAGES_ALLOWED_EXTRA_HTML_ATTRS"] = {
            "customtag": ["data-custom"]
        }

        sanitizer = DynamicSanitizedHTML()
        sample_html = '<customtag data-custom="value">Test</customtag>'
        result = sanitizer._deserialize(sample_html, None, None)

        assert '<customtag data-custom="value">Test</customtag>' in result



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-Pages is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Results for the requests service."""

from flask_sqlalchemy import Pagination
from invenio_records_resources.services.records.results import RecordItem, RecordList


class PageItem(RecordItem):
    """Single page result."""

    @property
    def data(self):
        """Property to get the page."""
        if self._data:
            return self._data

        self._data = self._schema.dump(
            self._obj,
            context={
                "identity": self._identity,
                "record": self._record,
            },
        )

        if self._links_tpl:
            self._data["links"] = self.links

        return self._data


class PageList(RecordList):
    """List of page results."""

    @property
    def hits(self):
        """Iterator over the hits."""
        for record in self._results:
            projection = self._schema.dump(
                record,
                context=dict(
                    identity=self._identity,
                    record=record,
                ),
            )

            if self._links_item_tpl:
                projection["links"] = self._links_item_tpl.expand(
                    self._identity, record
                )

            yield projection

    @property
    def total(self):
        """Get total number of pages."""
        return len(self._results)

    @property
    def _results(self):
        return (
            self._page_results.items
            if isinstance(self._page_results, Pagination)
            else self._page_results
        )

    def to_dict(self):
        """Return result as a dictionary."""
        hits = list(self.hits)

        res = {
            "hits": {
                "hits": hits,
                "total": self.total,
            }
        }

        if self._params:
            if self._links_tpl:
                res["links"] = self._links_tpl.expand(self._identity, self.pagination)

        return res

    @_results.setter
    def _results(self, value):
        self._page_results = value



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

import pytest
from invenio_records_resources.services.errors import PermissionDeniedError

from invenio_pages.records.errors import PageNotCreatedError, PageNotFoundError


def test_page_content(module_scoped_pages_fixture, base_client):
    """Test page content."""
    page = base_client.get("/pages/1")
    assert page.status_code == 200

    json = page.json
    json.pop("created")
    json.pop("updated")
    expected_data = {
        "title": "Page for Dogs!",
        "description": "",
        "url": "/dogs",
        "content": "Generic dog.",
        "id": "1",
        "template_name": "invenio_pages/default.html",
        "links": {"self": "https://127.0.0.1:5000/api/pages/1"},
    }
    assert json == expected_data


def test_html_content(module_scoped_pages_fixture, base_client):
    """Test page content."""
    page = base_client.get("/pages/4")
    assert page.status_code == 200

    json = page.json
    json.pop("created")
    json.pop("updated")
    expected_data = {
        "title": "Page for modern dogs!",
        "description": "",
        "url": "/htmldog",
        "content": "<h1>HTML aware dog.</h1>.\n" '<p class="test">paragraph<br /></p>',
        "id": "4",
        "template_name": "invenio_pages/default.html",
        "links": {"self": "https://127.0.0.1:5000/api/pages/4"},
    }
    assert json == expected_data


def test_search(module_scoped_pages_fixture, base_client):
    """Test search service function."""
    pages = base_client.get("/pages")
    assert pages.status_code == 200
    assert pages.json["hits"]["total"] == 4

    pages = base_client.get("/pages?size=2")
    assert pages.json["hits"]["total"] == 2

    pages = base_client.get("/pages?sort=title")
    assert pages.json["hits"]["hits"][0]["id"] == "3"

    pages = base_client.get("/pages?sort=title&sort_direction=desc")
    assert pages.json["hits"]["hits"][0]["id"] == "4"

    pages = base_client.get("/pages?sort_direction=desc")
    assert pages.json["hits"]["hits"][0]["id"] == "4"

    pages = base_client.get("/pages?size=3&sort=title&sort_direction=desc")
    assert pages.json["hits"]["hits"][0]["id"] == "4"

    pages = base_client.get("/pages?sort=url")
    assert pages.json["hits"]["hits"][0]["id"] == "3"
    assert pages.json["hits"]["hits"][3]["id"] == "4"

    pages = base_client.get("/pages?q=Generic%20dog")
    assert pages.json["hits"]["total"] == 1

    pages = base_client.get("/pages?q=dog&sort_direction=desc")
    assert pages.json["hits"]["total"] == 3
    assert pages.json["hits"]["hits"][0]["id"] == "4"


def test_create(module_scoped_pages_fixture, client, superuser):
    data = {
        "url": "/astures",
        "title": "Astures",
        "content": "Astures",
        "description": "Los astures (astures en latín) fueron un grupo de pueblos celtas...",
        "template_name": "invenio_pages/default.html",
    }
    superuser.login(client)

    page = client.post("/pages", json=data)
    assert page.json["title"] == "Astures"
    assert page.status_code == 201

    id = page.json["id"]
    assert client.get(f"/pages/{id}").json["title"] == "Astures"

    with pytest.raises(PageNotCreatedError):
        client.post("/pages", json=data)


def test_delete(module_scoped_pages_fixture, client, superuser):
    data = {
        "url": "/cantabros",
        "title": "Cantabros",
        "content": "Cantabros",
        "description": "El término cántabros...",
        "template_name": "invenio_pages/default.html",
    }
    superuser.login(client)

    page = client.post("/pages", json=data)
    id = page.json["id"]
    assert client.get(f"/pages/{id}").json["title"] == "Cantabros"

    client.delete(f"/pages/{id}")
    with pytest.raises(PageNotFoundError):
        client.get(f"/pages/{id}").json["title"] == "Cantabros"


def test_update(module_scoped_pages_fixture, client, admin):
    data = {
        "url": "/lusitanos",
        "title": "Lusitanos",
        "content": "Lusitanos",
        "description": "El término lusitanos...",
    }
    admin.login(client)

    assert client.get("/pages/1").json["title"] != "Lusitanos"

    client.put("/pages/1", json=data)

    json = client.get("/pages/1").json
    assert json["title"] == "Lusitanos"
    assert json["template_name"] == "invenio_pages/default.html"


def test_update_denied(module_scoped_pages_fixture, client):
    data = {
        "url": "/lusitanos",
        "title": "Lusitanos",
        "content": "Lusitanos",
        "description": "El término lusitanos...",
    }
    with pytest.raises(PermissionDeniedError):
        client.put("/pages/1", json=data)


def test_create_denied(module_scoped_pages_fixture, client, admin):
    data = {
        "url": "/arevacos",
        "title": "Arévacos",
        "content": "Arévacos",
        "description": "Los término arévacos",
        "template_name": "invenio_pages/default.html",
    }
    admin.login(client)
    with pytest.raises(PermissionDeniedError):
        client.post("/pages", json=data)


def test_delete_denied(module_scoped_pages_fixture, client, admin, superuser):
    data = {
        "url": "/numantinos",
        "title": "Numantinos",
        "content": "Numantinos",
        "description": "El término numantinos...",
        "template_name": "invenio_pages/default.html",
    }
    superuser.login(client)
    page = client.post("/pages", json=data)
    superuser.logout(client)

    id = page.json["id"]
    admin.login(client)

    with pytest.raises(PermissionDeniedError):
        client.delete(f"/pages/{id}")



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Views for Pages module."""


from flask import Blueprint, abort, current_app, g, render_template, request
from invenio_db import db
from sqlalchemy.orm.exc import NoResultFound
from werkzeug.exceptions import NotFound

from invenio_pages.proxies import current_pages_service

from .records.models import PageModel as Page

blueprint = Blueprint(
    "invenio_pages", __name__, url_prefix="/", template_folder="templates"
)


@blueprint.before_app_first_request
def register_pages():
    """Register URL rule of all static pages to the application."""
    # We need to set the function view, to be able to directly register the urls in the Flask.url_map
    current_app.view_functions["invenio_pages.view"] = view

    for page in Page.query.all():
        _add_url_rule(page.url)


@blueprint.app_template_filter("render_string")
def render_string(source):
    """Render a string in sandboxed environment.

    :param source: A string containing the page source.
    :returns: The rendered template.
    """
    return current_app.extensions["invenio-pages"].render_template(source)


def view():
    """Public interface to the page view.

    Models: `pages.pages`.
    Templates: Uses the template defined by the ``template_name`` field
    or ``pages/default.html`` if template_name is not defined.
    Context: page `pages.pages` object.
    """
    return render_page(request.path)  # pragma: no cover


def render_page(path):
    """Internal interface to the page view.

    :param path: Page path.
    :returns: The rendered template.
    """
    try:
        page = current_pages_service.read_by_url(g.identity, request.path).to_dict()
    except NoResultFound:
        abort(404)
    return render_template(
        [page["template_name"], current_app.config["PAGES_DEFAULT_TEMPLATE"]], page=page
    )


def handle_not_found(exception, **extra):
    """Custom blueprint exception handler."""
    assert isinstance(exception, NotFound)

    page = Page.query.filter(
        db.or_(Page.url == request.path, Page.url == request.path + "/")
    ).first()
    if page:
        _add_url_rule(page.url)
        return render_template(
            [page.template_name, current_app.config["PAGES_DEFAULT_TEMPLATE"]],
            page=page,
        )
    elif "wrapped" in extra:
        return extra["wrapped"](exception)
    else:
        return exception


def _add_url_rule(url):
    """Register URL rule to application URL map."""
    rule = current_app.url_rule_class(
        url,
        endpoint="invenio_pages.view",
        methods=["GET", "HEAD", "OPTIONS"],
        strict_slashes=True,
        merge_slashes=True,
    )
    current_app.url_map.add(rule)


def create_pages_api_bp(app):
    """Create the pages resource api blueprint."""
    ext = app.extensions["invenio-pages"]
    return ext.pages_resource.as_blueprint()



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Module tests."""

from flask import Flask

from invenio_pages import InvenioPages


def test_version():
    """Test version import."""
    from invenio_pages import __version__

    assert __version__


def test_init():
    """Test extension initialization."""
    app = Flask("testapp")
    InvenioPages(app)
    assert "invenio-pages" in app.extensions

    app = Flask("testapp")
    ext = InvenioPages()
    assert "invenio-pages" not in app.extensions
    ext.init_app(app)
    assert "invenio-pages" in app.extensions



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# invenio-administration is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio administration pages view module."""
from invenio_administration.views.base import (
    AdminResourceDetailView,
    AdminResourceEditView,
    AdminResourceListView,
)
from invenio_i18n import lazy_gettext as _


class PageListView(AdminResourceListView):
    """Configuration for pages list view."""

    api_endpoint = "/pages"
    name = "Pages"
    resource_config = "pages_resource"
    search_request_headers = {"Accept": "application/json"}
    title = "Pages"
    category = "Pages"
    pid_path = "id"
    icon = "file alternate outline"

    display_search = True
    display_delete = False
    display_create = False
    display_edit = True

    item_field_list = {
        "url": {"text": _("Url"), "order": 1, "width": 2},
        "title": {"text": _("Title"), "order": 2, "width": 3},
        "description": {"text": _("Description"), "order": 3, "width": 3},
        "created": {"text": _("Created"), "order": 4, "width": 3},
        "updated": {"text": _("Updated"), "order": 5, "width": 3},
    }

    search_config_name = "PAGES_SEARCH"
    search_facets_config_name = "PAGES_FACETS"
    search_sort_config_name = "PAGES_SORT_OPTIONS"


class PageEditView(AdminResourceEditView):
    """Configuration for page edit view."""

    name = "pages_edit"
    url = "/pages/<pid_value>/edit"
    resource_config = "pages_resource"
    pid_path = "id"
    api_endpoint = "/pages"
    title = "Edit Page"

    list_view_name = "Pages"

    form_fields = {
        "created": {"order": 1},
        "updated": {"order": 2},
        "url": {
            "order": 3,
            "text": _("URL"),
            "description": _("Relative path to the page."),
        },
        "title": {
            "order": 4,
            "text": _("Title"),
            "description": _("Title of the page."),
        },
        "description": {
            "order": 5,
            "text": _("Description"),
            "description": _("Description of the page"),
        },
        "content": {
            "order": 6,
            "text": _("Content"),
            "description": _("Content displayed by the page."),
            "rows": 10,
        },
    }


class PageDetailView(AdminResourceDetailView):
    """Configuration for page detail view."""

    url = "/pages/<pid_value>"
    api_endpoint = "/pages"
    name = "page-details"
    resource_config = "pages_resource"
    title = "Page"

    display_edit = True
    display_delete = False

    list_view_name = "Pages"
    pid_path = "id"

    item_field_list = {
        "created": {"text": _("Created"), "order": 1},
        "updated": {"text": _("Updated"), "order": 2},
        "url": {"text": _("Url"), "order": 3},
        "title": {"text": _("Title"), "order": 4},
        "description": {"text": _("Description"), "order": 5},
        "content": {"text": _("Content"), "order": 6, "escape": True},
    }
